<?php

/*
 * Plugin Name: WooCommerce - Модуль RUCODE
 * Description: Добавляет в WooCommerce возможность приёма платежей через платежную систему RUCODE
 * Author URI:  https://lumpx.com
 * Author: LumpX.com
 * Version: 1.0
*/

define('THIS_PLUGIN_DIR', dirname(__FILE__));

add_action('plugins_loaded', 'init_woo_rucode', 0);

function init_woo_rucode() {

	if (!class_exists('WC_Payment_Gateway')) return;

	class WC_Gateway_RUCODE extends WC_Payment_Gateway {
		
		function __construct() {
			$this->id = 'rucode';
			$this->has_fields         = false;
			$this->method_title       = __('RUCODE');
			$this->method_description = __('Приём платежей на сайте с помощью сервиса RUCODE<br>Установите WebHook в настройках: ?wc-api=WC_Gateway_RUCODE');

			$this->init_form_fields();
			$this->init_settings();

			$this->title = $this->get_option('title');
			$this->description   = $this->get_option('description');

			if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=') ) {
				add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
			} else {
				add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
			}

			add_action('woocommerce_api_wc_gateway_rucode', array(&$this, 'callback'));

		}
		
		function init_form_fields() {
			$this->form_fields = array(
				'enabled' => array(
					'title' => __('Enable/Disable', 'woocommerce'),
					'type' => 'checkbox',
					'default' => 'yes',
				),
				'title' => array(
					'title' => __('Title', 'woocommerce'),
					'type' => 'text',
					'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
					'default' => 'RUCODE',
					'desc_tip' => true,
				),
				'description' => [
					'title' => __('Description', 'woocommerce'),
					'type' => 'text',
					'description' => __( 'Приём платежей на сайте с помощью сервиса RUCODE', 'woocommerce' ),
					'default' => 'Оплата с помощью RUCODE'
				],
				'user_login' => array(
					'title' => 'Логин пользователя',
					'type' => 'text',
				),
				'secret_word' => array(
					'title' => 'API key',
					'type' => 'text',
				),
			);
		}
	
		function process_payment($order_id) {
			global $woocommerce;
			
			$order = new WC_Order($order_id);
			/*
			$url = ((!empty($_SERVER['HTTPS'])) ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

			$woocommerce->cart->empty_cart();
			return array(
				'result'   => 'success',
				'redirect' => $url.'&order=' . $order_id . '#rucode',
			);
			*/

			$amount = $order->get_total();
			$amount = str_replace(',', '.', $amount);
			$amount = number_format($amount, 2, '.', '');

			$url = 'https://rucode.cc/process?';

			$user_login = $this->get_option("user_login");
			$secret_word = $this->get_option("secret_word");

			$description = get_bloginfo('name').' – Заказ #'.$order_id;

			$currency = get_woocommerce_currency();

			$host = $_SERVER['HTTP_HOST'];
			
			$params = array(
				'login' => $user_login,
				'sum' => $amount,
				'curr' => $currency,
				'order' => $order_id,
				'desc' => $description,
				'success_url' => $order->get_checkout_order_received_url(),
				'sign' => md5($host.':'.$user_login.':'.(float)$amount.':'.$currency.':'.$secret_word.':'.(int)$order_id),
			);

			$url .= http_build_query($params, null, '&');

			$woocommerce->cart->empty_cart();
			/*
			$url = ((!empty($_SERVER['HTTPS'])) ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . '?order=' . $order_id . '#rucode';
			*/
			return array('result' => 'success', 'redirect' => $url);
		
		}

		public function callback() {
			$order_id = (int)$_POST['order'];
			
			if(empty($order_id)) {
				exit;
			}
			if(!isset($_POST['sign'])) {
				exit;
			}
			$order = new WC_Order($order_id);

			$amount = $order->get_total();
			$amount = str_replace(',', '.', $amount);
			$amount = number_format($amount, 2, '.', '');
			$amount_float = (float)$amount;
			
			$sign = $_POST['sign'];
			
			$user_login = $this->get_option("user_login");
			$secret_word = $this->get_option("secret_word");

			$check_sign2 = md5($user_login.':'.$order_id.':'.$amount_float.':'.$secret_word);

			if(hash_equals($sign, $check_sign2)) {
				$order->payment_complete();
			}
			exit;
		}
	}
}

function add_woo_rucode($methods) {
	$methods[] = 'WC_Gateway_RUCODE'; 
	return $methods;
}

add_filter('woocommerce_payment_gateways', 'add_woo_rucode');
